@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 
 if [ -z "$DD_API_KEY" ]
 then
@@ -10,4 +10,7 @@ DD_API_KEY=${DD_API_KEY} docker-compose up \
   --build \
   --abort-on-container-exit \
   --exit-code-from dd-opentracing-cpp-example
-docker rm dd-agent
+
+rcode=$?
+docker-compose rm --force dd-agent
+exit "$rcode"
