@@ -5,7 +5,10 @@
 
 #include <datadog/opentracing.h>
 
+#include <sstream>
+
 #include "agent_writer.h"
+#include "logger.h"
 #include "sample.h"
 #include "tracer.h"
 #include "tracer_options.h"
@@ -18,18 +21,20 @@ namespace opentracing {
 std::shared_ptr<ot::Tracer> makeTracer(const TracerOptions &options) {
   auto maybe_options = applyTracerOptionsFromEnvironment(options);
   if (!maybe_options) {
-    std::cerr << "Error applying TracerOptions from environment variables: "
-              << maybe_options.error() << std::endl
-              << "Tracer will be started without options from the environment" << std::endl;
+    std::ostringstream message;
+    message << "Error applying TracerOptions from environment variables: " << maybe_options.error()
+            << "\nTracer will be started without options from the environment\n";
+    StandardLogger(options.log_func).Log(LogLevel::error, message.str());
     maybe_options = options;
   }
   TracerOptions opts = maybe_options.value();
 
-  auto sampler = std::make_shared<RulesSampler>();
+  auto logger = makeLogger(opts);
+  auto sampler = std::make_shared<RulesSampler>(opts.sampling_limit_per_second);
   auto writer = std::shared_ptr<Writer>{
       new AgentWriter(opts.agent_host, opts.agent_port, opts.agent_url,
-                      std::chrono::milliseconds(llabs(opts.write_period_ms)), sampler)};
-  return std::shared_ptr<ot::Tracer>{new Tracer{opts, writer, sampler}};
+                      std::chrono::milliseconds(llabs(opts.write_period_ms)), sampler, logger)};
+  return std::shared_ptr<ot::Tracer>{new Tracer{opts, writer, sampler, logger}};
 }
 
 }  // namespace opentracing
