@@ -1,16 +1,82 @@
 #include "../src/tracer_factory.h"
 
+#include <datadog/tags.h>
+
+#include <catch2/catch.hpp>
+#include <cstdio>
+#include <fstream>
+#include <stdexcept>
+
 #include "../src/tracer.h"
 #include "mocks.h"
 
-// Source file needed to ensure compilation of templated class TracerFactory<MockTracer>
-#include <datadog/tags.h>
-
-#include <catch2/catch.hpp>
-
-#include "../src/tracer_factory.cpp"
 using namespace datadog::opentracing;
 
+namespace {
+
+// `EnvGuard` sets an environment variable to a specified value for the scope
+// of the `EnvGuard`, restoring the environment variable's previous value
+// afterward.
+class EnvGuard {
+  std::string name_;
+  bool had_value_;
+  std::string old_value_;
+
+ public:
+  explicit EnvGuard(std::string name, const char *value) : name_(name) {
+    if (const char *const old_value = std::getenv(name.c_str())) {
+      had_value_ = true;
+      old_value_ = old_value;
+    } else {
+      had_value_ = false;
+    }
+
+    const bool overwrite = true;
+    ::setenv(name.c_str(), value, overwrite);
+  }
+
+  ~EnvGuard() {
+    if (had_value_) {
+      const bool overwrite = true;
+      ::setenv(name_.c_str(), old_value_.c_str(), overwrite);
+    } else {
+      ::unsetenv(name_.c_str());
+    }
+  }
+};
+
+// `TmpFile` creates a temporary file having the specified content, and deletes
+// the file at the end of the `TmpFile` scope.  A path to the temporary file is
+// accesible via the `name` member function.
+class TmpFile {
+  std::string name_;
+
+ public:
+  explicit TmpFile(ot::string_view content) {
+    char name_buffer[L_tmpnam];
+    if (std::tmpnam(name_buffer) == nullptr) {
+      throw std::runtime_error("Unable to create a temporary file name.");
+    }
+    name_ = name_buffer;
+
+    std::ofstream file(name_buffer);
+    if (!file) {
+      throw std::runtime_error("unable to open temporary file for writing: " + name_);
+    }
+
+    file.write(content.data(), content.size());
+    if (!file) {
+      throw std::runtime_error("unable to write to temporary file named: " + name_);
+    }
+  }
+
+  ~TmpFile() { std::remove(name_.c_str()); }
+
+  const std::string &name() const { return name_; }
+};
+
+}  // namespace
+
 TEST_CASE("tracer factory") {
   TracerFactory<MockTracer> factory;
 
@@ -531,5 +597,88 @@ TEST_CASE("tracer factory") {
       REQUIRE(!tracer->opts.analytics_enabled);
       REQUIRE(std::isnan(tracer->opts.analytics_rate));
     }
+    SECTION("DD_SPAN_SAMPLING_RULES") {
+      const auto value = "[{}]";
+      EnvGuard guard("DD_SPAN_SAMPLING_RULES", value);
+
+      SECTION("overrides default") {
+        const auto input = R"json({
+          "service": "my-service"
+        })json";
+        std::string error;
+        const auto result = factory.MakeTracer(input, error);
+
+        REQUIRE(error == "");
+        REQUIRE(result);
+        REQUIRE(*result);
+        auto &tracer = static_cast<MockTracer &>(**result);
+        REQUIRE(tracer.opts.span_sampling_rules == value);
+      }
+
+      SECTION("overrides config") {
+        const auto input = R"json({
+          "service": "my-service",
+          "span_sampling_rules": [{}, {}]
+        })json";
+        std::string error;
+        const auto result = factory.MakeTracer(input, error);
+
+        REQUIRE(error == "");
+        REQUIRE(result);
+        REQUIRE(*result);
+        auto &tracer = static_cast<MockTracer &>(**result);
+        REQUIRE(tracer.opts.span_sampling_rules == value);
+      }
+    }
+    SECTION("DD_SPAN_SAMPLING_RULES_FILE") {
+      const std::string value = "[{}]";
+      TmpFile file(value);
+      EnvGuard guard("DD_SPAN_SAMPLING_RULES_FILE", file.name().c_str());
+
+      SECTION("overrides default") {
+        const auto input = R"json({
+          "service": "my-service"
+        })json";
+        std::string error;
+        const auto result = factory.MakeTracer(input, error);
+
+        REQUIRE(error == "");
+        REQUIRE(result);
+        REQUIRE(*result);
+        auto &tracer = static_cast<MockTracer &>(**result);
+        REQUIRE(tracer.opts.span_sampling_rules == value);
+      }
+
+      SECTION("overrides config") {
+        const auto input = R"json({
+          "service": "my-service",
+          "span_sampling_rules": [{}, {}]
+        })json";
+        std::string error;
+        const auto result = factory.MakeTracer(input, error);
+
+        REQUIRE(error == "");
+        REQUIRE(result);
+        REQUIRE(*result);
+        auto &tracer = static_cast<MockTracer &>(**result);
+        REQUIRE(tracer.opts.span_sampling_rules == value);
+      }
+
+      SECTION("does not override DD_SPAN_SAMPLING_RULES") {
+        const auto override_value = "[{}, {}, {}]";
+        EnvGuard guard("DD_SPAN_SAMPLING_RULES", override_value);
+        const auto input = R"json({
+          "service": "my-service"
+        })json";
+        std::string error;
+        const auto result = factory.MakeTracer(input, error);
+
+        REQUIRE(error == "");
+        REQUIRE(result);
+        REQUIRE(*result);
+        auto &tracer = static_cast<MockTracer &>(**result);
+        REQUIRE(tracer.opts.span_sampling_rules == override_value);
+      }
+    }
   }
 }
